unit game;

interface

uses
  Math, Classes, Dialogs, SysUtils, Forms, preferences;

const
  BoardWidth=8;
  NumberOfSquares=BoardWidth*BoardWidth div 2;

type
  TMove = record
    start: integer;
    dest: integer;
    kind: integer; {tyb tahu: 1 ... normal move
                               2 ... jump
                               3 ... promotion
                               4 ... jump & promotion
                               (5 ... queen's normal move)
                               (6 ... queen's jump)}
  end;

var CmpPlays: boolean;

//Pomocne funkce
function Sgn(a: integer):integer;
procedure CoordsToPlace(CoordX,CoordY:integer; var Place:integer);
procedure PlaceToCoords(Place:integer; var CoordX,CoordY:integer);
//Herni funkce
procedure NewGame(var CurrPos: array of integer;
                  var Player: integer);
procedure ClearPossibleMoves(var PossMoves: array of TMove);
procedure PossibleMoves(Player: integer;
                        var Position: array of integer;
                        var PossMoves: array of TMove;
                        var NPossMoves: integer);
procedure DoMove(Move:TMove;
                 var Position: array of integer;
                 var Player:integer);
procedure GameOver(var CurrPos: array of integer;
                   var NPossMoves: integer;
                   var GameState: integer);
//Premysleci funkce
function ChooseMove(Position: array of integer;
                    Player: integer;
                    Depth: integer): TMove;




//------------------------------------------------------------------------------
implementation

//Pomocne funkce

//------------------------------------------------------------------------------

function Sgn(a: integer):integer;
begin
  if a > 0 then Sgn:=1 else if a = 0 then Sgn:=0 else Sgn:=-1;
end;

//------------------------------------------------------------------------------

procedure CoordsToPlace(CoordX,CoordY:integer; var Place:integer);
begin
  Place:=((CoordY-1)*(BoardWidth div 2))+((CoordX+1) div 2)
end;

//------------------------------------------------------------------------------

procedure PlaceToCoords(Place:integer; var CoordX,CoordY:integer);
begin
  CoordY:=(Place+3) div (BoardWidth div 2);
  CoordX:=(((Place-1) mod (BoardWidth div 2))+1)*2 - CoordY mod 2;
end;

//------------------------------------------------------------------------------

//Herni funkce

//------------------------------------------------------------------------------

procedure NewGame(var CurrPos: array of integer;
                  var Player:integer);

var i: integer;
begin
  for i:=1 to 3*(BoardWidth div 2) do
    CurrPos[i]:=1;
  for i:=3*(BoardWidth div 2)+1 to (BoardWidth-3)*(BoardWidth div 2) do
    CurrPos[i]:=0;
  for i:=(BoardWidth-3)*(BoardWidth div 2)+1 to NumberOfSquares do
    CurrPos[i]:=-1;
  Player:=1;
  if White>0 then DoMove(ChooseMove(CurrPos, Player, 2*White), CurrPos, Player);
end;

//------------------------------------------------------------------------------

procedure ClearPossibleMoves(var PossMoves: array of TMove);

var i:integer;
begin
  for i:=0 to 6*BoardWidth do begin
    PossMoves[i].start:=0;
    PossMoves[i].dest:=0;
    PossMoves[i].kind:=0;
  end;
end;

//------------------------------------------------------------------------------

procedure PossibleMoves(Player: integer;
                        var Position: array of integer;
                        var PossMoves: array of TMove;
                        var NPossMoves: integer);

var i,j:integer;
    CoordX,CoordY: integer;
    PlaceStart,PlaceDest,PlaceSkipped :integer;
    jump: boolean;

begin
  j:=0;
  jump:=false;
  for PlaceStart:=1 to NumberOfSquares do begin
    if Player*Position[PlaceStart] > 0 then begin
      PlaceToCoords(PlaceStart,CoordX,CoordY);
      // 1) figure is a pawn
      if Abs(Position[PlaceStart])=1 then begin

        // a) normal move
        if not(jump) and
        (CoordY+Player < BoardWidth) and
        (CoordY+Player > 1) then begin

          // Right
          CoordsToPlace(CoordX+1, CoordY+Player, PlaceDest);
          if (CoordX+1 <= BoardWidth) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=1;
          end;

          // Left
          CoordsToPlace(CoordX-1, CoordY+Player, PlaceDest);
          if (CoordX-1 >= 1) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=1;
          end;

        end else

        // b) promotion
        if not(jump) and
        ((CoordY+Player = BoardWidth) or (CoordY+Player = 1)) then begin

          // Right
          CoordsToPlace(CoordX+1, CoordY+Player, PlaceDest);
          if (CoordX+1 <= BoardWidth) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=3;
          end;

          //Left
          CoordsToPlace(CoordX-1, CoordY+Player, PlaceDest);
          if (CoordX-1 >= 1) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=3;
          end;
        end;

        // c) jump
        if (CoordY+2*Player < BoardWidth) and
        (CoordY+2*Player > 1) then begin

          // Right
          CoordsToPlace(CoordX+2, CoordY+2*Player, PlaceDest);
          CoordsToPlace(CoordX+1, CoordY+Player, PlaceSkipped);
          if (CoordX+2 <= BoardWidth) and
          (Sgn(Position[PlaceSkipped])=-Player) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=2;
          end;

          // Left
          CoordsToPlace(CoordX-2, CoordY+2*Player, PlaceDest);
          CoordsToPlace(CoordX-1, CoordY+Player, PlaceSkipped);
          if (CoordX-2 >= 1) and
          (Sgn(Position[PlaceSkipped])=-Player) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=2;
          end;

        end else

        // d) jump promotion
        if (CoordY+2*Player = BoardWidth) or
        (CoordY+2*Player = 1) then begin

          // Right
          CoordsToPlace(CoordX+2, CoordY+2*Player, PlaceDest);
          CoordsToPlace(CoordX+1, CoordY+Player, PlaceSkipped);
          if (CoordX+2 <= BoardWidth) and
          (Sgn(Position[PlaceSkipped])=-Player) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=4;
          end;

          //Left
          CoordsToPlace(CoordX-2, CoordY+2*Player, PlaceDest);
          CoordsToPlace(CoordX-1, CoordY+Player, PlaceSkipped);
          if (CoordX-2 >= 1) and
          (Sgn(Position[PlaceSkipped])=-Player) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=4;
          end;
        end;
      // 2. Figure is a queen
      end else if Abs(Position[PlaceStart])=2 then begin
        // a) normal move
        if not(jump) and (CoordY+1 <= BoardWidth) then begin

          // up right
          CoordsToPlace(CoordX+1, CoordY+1, PlaceDest);
          if (CoordX+1 <= BoardWidth) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=1;
          end;

          // up left
          CoordsToPlace(CoordX-1, CoordY+1, PlaceDest);
          if (CoordX-1 >= 1) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=1;
          end;
        end;
        if not(jump) and (CoordY-1 >= 1) then begin

          // down right
          CoordsToPlace(CoordX+1, CoordY-1, PlaceDest);
          if (CoordX+1 <= BoardWidth) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=1;
          end;

          // down left
          CoordsToPlace(CoordX-1, CoordY-1, PlaceDest);
          if (CoordX-1 >= 1) and (Position[PlaceDest]=0) then begin
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=1;
          end;
        end;

        //b) jump
        if (CoordY+2 <= BoardWidth) then begin

          // up right
          CoordsToPlace(CoordX+2, CoordY+2, PlaceDest);
          CoordsToPlace(CoordX+1, CoordY+1, PlaceSkipped);
          if (CoordX+2 <= BoardWidth) and
          (Sgn(Position[PlaceSkipped])=-Sgn(Player)) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=2;
          end;

          //up left
          CoordsToPlace(CoordX-2, CoordY+2, PlaceDest);
          CoordsToPlace(CoordX-1, CoordY+1, PlaceSkipped);
          if (CoordX-2 >= 1) and
          (Sgn(Position[PlaceSkipped])=-Sgn(Player)) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=2;
          end;
        end;

        if (CoordY-2 >= 1 ) then begin

          // down right
          CoordsToPlace(CoordX+2, CoordY-2, PlaceDest);
          CoordsToPlace(CoordX+1, CoordY-1, PlaceSkipped);
          if (CoordX+2 <= BoardWidth) and
          (Sgn(Position[PlaceSkipped])=-Sgn(Player)) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=2;
          end;

          //up left
          CoordsToPlace(CoordX-2, CoordY-2, PlaceDest);
          CoordsToPlace(CoordX-1, CoordY-1, PlaceSkipped);
          if (CoordX-2 >= 1) and
          (Sgn(Position[PlaceSkipped])=-Sgn(Player)) and
          (Position[PlaceDest]=0) then begin
            if not(jump) then begin
              for i:=0 to j do begin
                PossMoves[i].start:=0;
                PossMoves[i].dest:=0;
                PossMoves[i].kind:=0;
              end;
              j:=0;
              jump:=true;
            end;
            j:=j+1;
            PossMoves[j].start:=PlaceStart;
            PossMoves[j].dest:=PlaceDest;
            PossMoves[j].kind:=2;
          end;
        end;
      end;
    end;
  end;
  NPossMoves:=j;
end; {of procedure PossibleMoves}

//------------------------------------------------------------------------------

procedure DoMove(Move: TMove;
                 var Position: array of integer;
                 var Player: integer);
var i: integer;
    Place,CoordXstart,CoordYstart,
    CoordXdest,CoordYdest: integer;
    PossMoves: array[0..6*NumberOfSquares] of TMove;
    NPossMoves: integer;
    ChangePlayer: boolean;

begin
//  ShowMessage(IntToStr(Move.kind));
  case Move.kind of
    1:  begin
          Position[Move.dest]:=Position[Move.start];
          Position[Move.start]:=0;
          Player:=-Player;
        end;

    2:  begin
          PlaceToCoords(Move.start, CoordXstart, CoordYstart);
          PlaceToCoords(Move.dest, CoordXdest, CoordYdest);
          CoordsToPlace((CoordXstart+CoordXdest) div 2,
                        (CoordYstart+CoordYdest) div 2,
                        Place);
          Position[Move.dest]:=Position[Move.start];
          Position[Move.start]:=0;
          Position[Place]:=0;
          PossibleMoves(Player, Position, PossMoves, NPossMoves);
          ChangePlayer:=true;
          for i:=1 to NPossMoves do begin
            if ((PossMoves[i].start = Move.dest) and
            ((PossMoves[i].kind = 2) or
            (PossMoves[i].kind = 4) or
            (PossMoves[i].kind = 4))) then begin
              ChangePlayer:=false;
            end;
          end;
          if ChangePlayer then Player:=-Player
        end;

    3:  begin
          Position[Move.dest]:=Position[Move.start]*2;
          Position[Move.start]:=0;
          Player:=-Player;
        end;

    4:  begin
          PlaceToCoords(Move.start, CoordXstart, CoordYstart);
          PlaceToCoords(Move.dest, CoordXdest, CoordYdest);
          CoordsToPlace((CoordXstart+CoordXdest) div 2,
                        (CoordYstart+CoordYdest) div 2,
                        Place);
          Position[Move.dest]:=Position[Move.start]*2;
//        ShowMessage('Position[Move.dest] = '+IntToStr(Position[Move.dest]));
          Position[Move.start]:=0;
          Position[Place]:=0;
          Player:=-Player
        end;
  end;
end;

//------------------------------------------------------------------------------

procedure GameOver(var CurrPos: array of integer;
                   var NPossMoves: integer;
                   var GameState: integer);
var i,BlackPieces,WhitePieces:integer;
begin
  BlackPieces:=0;
  WhitePieces:=0;
  if NPossMoves = 0 then begin
    for i:=1 to NumberOfSquares do begin
      if CurrPos[i] > 0 then WhitePieces:=WhitePieces+1
      else if CurrPos[i] < 0 then BlackPieces:=BlackPieces+1;

      if BlackPieces = 0 then GameState:=1      //white wins
      else if WhitePieces = 0 then GameState:=2 //black wins
      else GameState:=3;                        //draw
    end;
  end;
end;

//------------------------------------------------------------------------------

//Premysleci funkce

//------------------------------------------------------------------------------

function RatePosition(Position: array of integer): real;
var
  PossMoves: array[0..6*BoardWidth] of TMove;
  i,j,NPieces,BlackMoves,WhiteMoves: integer;
  CoordXW, CoordYW: integer;
  CoordXB, CoordYB: integer;
  Dist, MinDist: integer;
  BlackPieces,WhitePieces,Rating: real;

begin
  NPieces:=0;
  WhitePieces:=0;
  BlackPieces:=0;
  WhiteMoves:=0;
  BlackMoves:=0;
  for i:=1 to NumberOfSquares do begin
    if Position[i]=1 then WhitePieces:=WhitePieces+1;
    if Position[i]=2 then WhitePieces:=WhitePieces+1.9;
    if Position[i]=-1 then BlackPieces:=BlackPieces-1;
    if Position[i]=-2 then BlackPieces:=BlackPieces-1.9;
    NPieces:=NPieces+1;
  end;
  Rating:=2*(WhitePieces+BlackPieces);

  ClearPossibleMoves(PossMoves);
  PossibleMoves(1, Position, PossMoves, WhiteMoves);
  if (PossMoves[1].kind=2) or (PossMoves[1].kind=4) then Rating:=Rating+1.6;
  for i:=1 to WhiteMoves do
    if PossMoves[i].kind > 2 then Rating:=Rating+0.8;

  ClearPossibleMoves(PossMoves);
  PossibleMoves(-1, Position, PossMoves, BlackMoves);
  if (PossMoves[1].kind=2) or (PossMoves[1].kind=4) then Rating:=Rating-1.6;
  for i:=1 to BlackMoves do
    if PossMoves[i].kind > 2 then Rating:=Rating-0.8;

  if WhitePieces > BlackPieces then Rating:=Rating+1/NPieces;
  if BlackPieces > WhitePieces then Rating:=Rating-1/NPieces;

{  MinDist:=1000000; {nekonecno
  if WhitePieces=1.9 then begin
    for i:=1 to NumberOfSquares do
      if Position[i] = 2 then PlaceToCoords(i, CoordXW, CoordYW);
    for i:=1 to NumberOfSquares do begin
      if Position[i] < 0 then PlaceToCoords(i, CoordXB, CoordYB);
      Dist:=Abs(CoordXW-CoordXB)+Abs(CoordYW-CoordYB);
      if Dist < MinDist then MinDist:=Dist;
    end;
    Rating:=MinDist;
  end;}


  RatePosition:=Rating+Random/(2*NumberOfSquares);
end;

//------------------------------------------------------------------------------

function RatePositionR(Position: array of integer;
                       Player: integer;
                       Depth: integer): real;
var
  Move: TMove;
  PossMoves: array[0..6*NumberOfSquares] of TMove;
  NPossMoves,GameState: integer;

begin
  PossibleMoves(Player, Position, PossMoves, NPossMoves);
  if NPossMoves = 0 then begin
    GameOver(Position, NPossMoves, GameState);
    if GameState=3 then RatePositionR:=0
    else if GameState=2 then RatePositionR:=-1000000
    else if GameState=1 then RatePositionR:= 1000000;
  end else begin
    if Depth = 0 then RatePositionR:=RatePosition(Position)
    else begin
      Depth:=Depth-1;
      Move:=ChooseMove(Position, Player, Depth);
      DoMove(Move, Position, Player);
      RatePositionR:=RatePositionR(Position, Player, Depth);
    end;
  end;
end;

//------------------------------------------------------------------------------

function RateMove(Position: array of integer;
                  Player: integer;
                  Move: TMove;
                  Depth: integer): real;
var Pl: integer;
begin
  Pl:=Player;
  DoMove(Move, Position, Pl);
  RateMove:=Player*RatePositionR(Position,Pl,Depth);
end;

//------------------------------------------------------------------------------

function ChooseMove(Position: array of integer;
                    Player: integer;
                    Depth: integer): TMove;
var
  PossMoves: array[0..6*BoardWidth] of TMove;
  BestIndex,i,NPossMoves: integer;
  Rating, BestRating: real;
begin
  BestRating:=-200;
  BestIndex:=1;
  PossibleMoves(Player, Position, PossMoves, NPossMoves);
  for i:=1 to NPossMoves do begin
    Rating:=RateMove(Position, Player, PossMoves[i], Depth);
    if Rating > BestRating then begin
      BestRating:=Rating;
      BestIndex:=i;
    end;
  end;
  Application.ProcessMessages;
  ChooseMove:=PossMoves[BestIndex];
end;

//------------------------------------------------------------------------------



end.
