unit intrface;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Math, ExtCtrls, Menus, game;

type
  TfrmMain = class(TForm)
    imgBoard: TImage;
    MainMenu: TMainMenu;
    menuGame: TMenuItem;
    menuNew: TMenuItem;
    menuExit: TMenuItem;
    menuOptions: TMenuItem;
    menuHelp: TMenuItem;
    menuPreferences: TMenuItem;
    menuAbout: TMenuItem;
    procedure SayGameOver(GameState: integer);
    procedure DrawPiece(where:integer; piece:integer);
    procedure DrawBoard(Sender: TObject);
    procedure menuExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure imgBoardMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgBoardMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure imgBoardMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure menuNewClick(Sender: TObject);
    procedure menuPreferencesClick(Sender: TObject);
    procedure menuAboutClick(Sender: TObject);
    private
      { Private declarations }
    public
      { Public declarations }
  end;

var
  frmMain: TfrmMain;
  CurrPos, Pos: array[0..NumberOfSquares] of integer;
  MsDown: boolean;
  Move: TMove;
  PossMoves: array[0..6*BoardWidth] of TMove;
  Player: integer;
  NPossMoves: integer;
  GameState: integer;
  ActPlace: integer;
  NBlink:integer;

//----------------------------=IMPLEMENTATION=----------------------------------
implementation

uses preferences, about;

{$R *.DFM}
//------------------------------------------------------------------------------

procedure TfrmMain.SayGameOver(GameState: integer);
var Again: boolean;
begin
  Again:=false;
  if GameState = 1 then begin
    imgBoard.Enabled:=false;
    if MessageBox(frmMain.ClientHandle,
                  'White wins. Play again?',
                  'Game over',
                  MB_YESNO+MB_DEFBUTTON2+MB_ICONQUESTION) = IDYES then
      Again:=true;
  end;

  if GameState = 2 then begin
    imgBoard.Enabled:=false;
    if MessageBox(frmMain.ClientHandle,
                  'Black wins. Play again?',
                  'Game over',
                  MB_YESNO+MB_DEFBUTTON2+MB_ICONQUESTION) = IDYES then
      Again:=true;
  end;

  if GameState = 3 then begin
    imgBoard.Enabled:=false;
    if MessageBox(frmMain.ClientHandle,
                  'Draw. Play again?',
                  'Game over',
                  MB_YESNO+MB_DEFBUTTON2+MB_ICONQUESTION) = IDYES then
      Again:=true;
  end;

  if Again then begin
    imgBoard.Enabled:=true;
    frmMain.FormCreate(frmMain);
  end else
    Application.Terminate;

end;

//------------------------------------------------------------------------------

procedure TfrmMain.DrawPiece(where: integer; piece:integer);

var CoordX,CoordY,SquareWidth: integer;
begin
  PlaceToCoords(where,CoordX,CoordY);
  with imgBoard.Canvas do begin
    if piece > 0 then begin
      Brush.Color:=clWhite;
      Pen.Color:=clBlack;
    end else begin
      Brush.Color:=clBlack;
      Pen.Color:=clWhite;
    end;
    SquareWidth:=imgBoard.Width div (BoardWidth+2);
    if Abs(piece)=1 then begin // draw pawn
      Ellipse (CoordX*SquareWidth+4, imgBoard.Width-(CoordY+1)*SquareWidth+4,
              (CoordX+1)*SquareWidth-4, imgBoard.Width-CoordY*SquareWidth-4  );
    end;

    if Abs(piece)=2 then begin // draw queen
      Ellipse (CoordX*SquareWidth+4, imgBoard.Width-(CoordY+1)*SquareWidth+4,
              (CoordX+1)*SquareWidth-4, imgBoard.Width-CoordY*SquareWidth-4  );
      Ellipse (CoordX*SquareWidth+8, imgBoard.Width-(CoordY+1)*SquareWidth+8,
              (CoordX+1)*SquareWidth-8, imgBoard.Width-CoordY*SquareWidth-8  );
    end;

    if piece=3 then begin // draw selection
      Brush.Color:=clGray;
      Pen.Color:=clYellow;
      Rectangle (CoordX*SquareWidth+4, imgBoard.Width-(CoordY+1)*SquareWidth+4,
             (CoordX+1)*SquareWidth-4, imgBoard.Width-CoordY*SquareWidth-4  );
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TfrmMain.DrawBoard(Sender: TObject);

var i,j, SquareWidth: integer;
begin
  with imgBoard.Canvas do begin
    Brush.Color:=clDkGray;
    Pen.Color:=clWhite;
    Rectangle(0, 0, frmMain.ClientHeight, frmMain.ClientWidth);
    Pen.Style:=psSolid;
    Pen.Color:=clBlack;
    Pen.Width:=3;
    SquareWidth:=imgBoard.Width div (BoardWidth+2);

    for i:=1 to BoardWidth+1 do begin
      MoveTo(SquareWidth, SquareWidth*i);
      LineTo(SquareWidth*(BoardWidth+1), SquareWidth*i);
      MoveTo(SquareWidth*i, SquareWidth);
      LineTo(SquareWidth*i, SquareWidth*(BoardWidth+1));
    end;

    Pen.Width:=0;

    for i:=2 to BoardWidth+1 do begin
      for j:=2 to BoardWidth+1 do begin
        if ((i+j+1) mod 2) = 0 then begin
          Brush.Color:=clGray;
          Rectangle((i-1)*SquareWidth+1,(j-1)*SquareWidth+1,
                        i*SquareWidth,    j*SquareWidth);
        end else begin
          Brush.Color:=clWhite;
          Rectangle((i-1)*SquareWidth+1,(j-1)*SquareWidth+1,
                        i*SquareWidth,    j*SquareWidth)
        end
      end;
    end;

    for i:=1 to NumberOfSquares do begin
      if CurrPos[i]<>0 then DrawPiece(i, CurrPos[i]);
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TfrmMain.menuExitClick(Sender: TObject);
begin
  Application.Terminate;
end;

//------------------------------------------------------------------------------

procedure TfrmMain.FormCreate(Sender: TObject);

begin
  GameState:=0;
  NewGame(CurrPos, Player);
//  if White>0 then DoMove(ChooseMove(CurrPos, Player, 2*White), CurrPos, Player);
  DrawBoard(frmMain);
end;

//------------------------------------------------------------------------------

procedure TfrmMain.imgBoardMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);

var CoordX, CoordY, Place, SquareWidth: integer; i:integer;
begin
  MsDown:=true;
  SquareWidth:=imgBoard.Width div (BoardWidth+2);
  CoordX:=X div SquareWidth;
  CoordY:=(imgBoard.Width - Y) div SquareWidth;
  if (CoordX+CoordY) mod 2 = 0 then
    Place:=(CoordY-1)*(BoardWidth div 2) + (CoordX+1) div 2
  else Place:=100;
  Move.start:=100;
  for i:=1 to 6*BoardWidth do
    if PossMoves[i].start = Place then begin
      DrawPiece(PossMoves[i].dest, 3);
      Move.start:=Place;
    end;
end;

//------------------------------------------------------------------------------

procedure TfrmMain.imgBoardMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);

var CoordX,CoordY,Place,SquareWidth: integer; Moveable: boolean; i:integer;
begin
  SquareWidth:=imgBoard.Width div (BoardWidth+2);
  CoordX:=X div SquareWidth;
  CoordY:=(imgBoard.Width - Y) div SquareWidth;

  if ((CoordX+CoordY) mod 2 = 0) and
  (CoordX > 0) and
  (CoordX < BoardWidth+1) and
  (CoordY < BoardWidth+1) and
  (CoordY > 0) then
    Place:=(CoordY-1)*(BoardWidth div 2) + (CoordX+1) div 2
  else
    Place:=100;

  Moveable:=false;
  PossibleMoves(Player, CurrPos, PossMoves, NPossMoves);

  for i:=1 to 6*BoardWidth do
    if PossMoves[i].start = Place then Moveable:=true;

  if Moveable then
    frmMain.Cursor:=crHandPoint
  else
    frmMain.Cursor:=crDefault;
end;

//------------------------------------------------------------------------------

procedure TfrmMain.imgBoardMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var CoordX, CoordY, Place, SquareWidth, i:integer;
begin
  SquareWidth:=imgBoard.Width div (BoardWidth+2);
  CoordX:=X div SquareWidth;
  CoordY:=(imgBoard.Width - Y) div SquareWidth;
  if (CoordX+CoordY) mod 2 = 0 then
    Place:=(CoordY-1)*(BoardWidth div 2) + (CoordX+1) div 2
  else Place:=100;
  if MsDown then begin
    for i:=1 to 6*BoardWidth do begin
      if (PossMoves[i].start = Move.start) and
      (PossMoves[i].dest = Place) then begin
        Move.dest:=Place;
        Move.kind:=PossMoves[i].kind;
        DoMove(Move, CurrPos, Player);
        ClearPossibleMoves(PossMoves);
      end;
    end;

    MsDown:=false;

    DrawBoard(imgBoard);

    PossibleMoves(Player, CurrPos, PossMoves, NPossMoves);
    GameOver(CurrPos, NPossMoves, GameState);


    if (((Player=1) and (White>0)) or ((Player=-1) and (Black>0))) and
    (GameState=0) then begin
      CmpPlays:=true;
      Application.ProcessMessages;
      imgBoard.Enabled:=false;

      PossibleMoves(Player, CurrPos, PossMoves, NPossMoves);

      if Player=1 then begin
        while Player=1 do
          DoMove(ChooseMove(CurrPos, Player, 2*White), CurrPos, Player);
          DrawBoard(frmMain);
      end else begin
        while Player=-1 do
          DoMove(ChooseMove(CurrPos, Player, 2*Black), CurrPos, Player);
          DrawBoard(frmMain);
      end;
      PossibleMoves(Player, CurrPos, PossMoves, NPossMoves);
      GameOver(CurrPos, NPossMoves, GameState);
      imgBoard.Enabled:=true;
      DrawBoard(imgBoard);
      if GameState > 0 then SayGameOver(GameState);
    end;
    if GameState > 0 then SayGameOver(GameState);
    MsDown:=false;
  end;

end;

//------------------------------------------------------------------------------

procedure TfrmMain.menuNewClick(Sender: TObject);
begin
  NewGame(CurrPos, Player);
  DrawBoard(imgBoard);
end;

//------------------------------------------------------------------------------

procedure TfrmMain.menuPreferencesClick(Sender: TObject);
begin
  frmPreferences.Left:=frmMain.Left+((frmMain.Width-frmPreferences.Width) div 2);
  frmPreferences.Top:=frmMain.Top+((frmMain.Height-frmPreferences.Height) div 2);
  frmPreferences.ShowModal;
end;

//------------------------------------------------------------------------------

procedure TfrmMain.menuAboutClick(Sender: TObject);
begin
  frmAbout.Left:=frmMain.Left+((frmMain.Width-frmAbout.Width) div 2);
  frmAbout.Top:=frmMain.Top+((frmMain.Height-frmAbout.Height) div 2);
  frmAbout.ShowModal;
end;

//------------------------------------------------------------------------------


end.
